/*
 * USB-MIDI class driver for GR-SAKURA
 * Copyright 2012-2014 Yuuichi Akagawa
 *
 *******************************************************************************
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 *******************************************************************************
 */
#if !defined(_USBH_MIDI_H_)
#define _USBH_MIDI_H_
#include <rxduino.h>
#include "tkusbhost.h"

//USB Class
#define USB_CLASS_AUDIO                0x01
#define USB_SUBCLASS_MIDISTREAMING     0x03

#define MIDI_MAX_ENDPOINTS    5 //endpoint 0, bulk_IN(MIDI), bulk_OUT(MIDI), bulk_IN(VSP), bulk_OUT(VSP)
#define DESC_BUFF_SIZE        256
#define MIDI_EVENT_PACKET_SIZE 64

struct EPinfo
{
    byte epAddr;        // Endpoint address 
    word maxPktSize;    // Maximum packet size
    byte bInterval;     // Polling interval time
};

class USBH_MIDI
{
private:
    static const byte epDataInIndex;            // DataIn endpoint index(MIDI)
    static const byte epDataOutIndex;           // DataOUT endpoint index(MIDI)
    static const byte epDataInIndexVSP;         // DataIn endpoint index(Vendor Specific Protocl)
    static const byte epDataOutIndexVSP;        // DataOUT endpoint index(Vendor Specific Protocl)
    static const int  usbTimeout;
  
    byte bConfNum;
    bool isPollEnable;
    bool isMidiFound;
    bool isLedEnable;

    // Endpoint data structure
    EPinfo epInfo[MIDI_MAX_ENDPOINTS];
    // MIDI Event packet buffer
    byte recvBuf[MIDI_EVENT_PACKET_SIZE];
    byte readPtr;

    // private functions
    void Init();
    void Release();
    bool parseConfigDescr(byte conf);
    int lookupMsgSize(uint8_t midiMsg);
    int countSysExDataSize(byte *dataptr);

public:
    USBH_MIDI(bool enableLED = false);
    bool isConnected();
    int  rawRead(byte *dataptr);
    int  read(byte *dataptr);
    int  write(byte *dataptr, byte nCable=0);
    int  writeSysEx(byte *dataptr, word datasize, byte nCable=0);

    //Utility function
    inline word getVendorId(void){return USBHostInfo.DeviceDesc.idVendor;};
    inline word getProductId(void){return USBHostInfo.DeviceDesc.idProduct;};
    inline byte getInterval(void){return epInfo[epDataInIndex].bInterval;};
};
#endif //_USBH_MIDI_H_
